from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from beanie.odm.documents import Document


def get_dict(document: "Document"):
    exclude = None if document.id is not None else {"id"}
    return document.dict(by_alias=True, exclude=exclude)



from typing import Type, TYPE_CHECKING, Any, Mapping

from pymongo.results import DeleteResult

from beanie.odm.interfaces.session import SessionMethods

if TYPE_CHECKING:
    from beanie.odm.documents import DocType


class DeleteQuery(SessionMethods):
    """
    Deletion Query
    """

    def __init__(
        self,
        document_model: Type["DocType"],
        find_query: Mapping[str, Any],
    ):
        self.document_model = document_model
        self.find_query = find_query
        self.session = None


class DeleteMany(DeleteQuery):
    def __await__(self) -> DeleteResult:
        """
        Run the query
        :return:
        """
        yield from self.document_model.get_motor_collection().delete_many(
            self.find_query, session=self.session
        )


class DeleteOne(DeleteQuery):
    def __await__(self) -> DeleteResult:
        """
        Run the query
        :return:
        """
        yield from self.document_model.get_motor_collection().delete_one(
            self.find_query, session=self.session
        )



from beanie.migrations.controllers.free_fall import free_fall_migration
from beanie.migrations.controllers.iterative import iterative_migration
from beanie.odm.fields import PydanticObjectId, Indexed
from beanie.odm.utils.general import init_beanie
from beanie.odm.documents import Document

__version__ = "1.2.1"
__all__ = [
    # ODM
    "Document",
    "init_beanie",
    "PydanticObjectId",
    "Indexed",
    # Migrations
    "iterative_migration",
    "free_fall_migration",
]



from abc import abstractmethod
from typing import (
    Optional,
    List,
    TypeVar,
    Type,
    Dict,
    Any,
    Generic,
    cast,
)

from pydantic.main import BaseModel

CursorResultType = TypeVar("CursorResultType")


class BaseCursorQuery(Generic[CursorResultType]):
    """
    BaseCursorQuery class. Wrapper over AsyncIOMotorCursor,
    which parse result with model
    """

    @abstractmethod
    def get_projection_model(self) -> Optional[Type[BaseModel]]:
        ...

    @property
    @abstractmethod
    def motor_cursor(self):
        ...

    def __aiter__(self):
        return self

    async def __anext__(self) -> CursorResultType:
        if getattr(self, "cursor", None) is None:
            self.cursor = self.motor_cursor
        next_item = await self.cursor.__anext__()
        projection = self.get_projection_model()
        return (
            projection.parse_obj(next_item)
            if projection is not None
            else next_item
        )

    async def to_list(
        self, length: Optional[int] = None
    ) -> List[CursorResultType]:  # noqa
        """
        Get list of documents

        :param length: Optional[int] - length of the list
        :return: Union[List[BaseModel], List[Dict[str, Any]]]
        """
        motor_list: List[Dict[str, Any]] = await self.motor_cursor.to_list(
            length
        )
        projection = self.get_projection_model()
        if projection is not None:
            return cast(
                List[CursorResultType],
                [projection.parse_obj(i) for i in motor_list],
            )
        return cast(List[CursorResultType], motor_list)
